using Newtonsoft.Json;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using AdHocApi;
using AdHocApi.BusinessLayer;
using AdHocApi.Models;
using AdHocApi.ViewModels;
using AdHocApi.Domain;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace AdHoc.Controllers
{
     /// <summary>
     /// Works like the Base class for CategoryController class.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the CategoryController class under the Controller folder. **
     /// *************************************************************************************************************
     /// </summary>
     public partial class CategoryController : Controller
     {
         private Category _category;
         private CategoryViewModel _categoryViewModel;
         private CategoryForeachViewModel _categoryForEachViewModel;
         private readonly ICategoryBusinessLayer _categoryBusinessLayer;

         // constructor
         public CategoryController(Category category, ICategoryBusinessLayer categoryBusinessLayer, CategoryViewModel categoryViewModel, CategoryForeachViewModel categoryForeachViewModel)
         {
             _category = category;
             _categoryViewModel = categoryViewModel;
             _categoryForEachViewModel = categoryForeachViewModel;
             _categoryBusinessLayer = categoryBusinessLayer;
         }

         #region actions used by their respective views

         /// <summary>
         /// GET: /Category/
         /// Gets the view used by the Index razor view
         /// </summary>
         public IActionResult Index()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Category/Add
         /// Gets the view model used by the Add razor view
         /// </summary>
         public IActionResult Add(string urlReferrer = null)
         {
             // return the view model
             return this.GetAddViewModel(urlReferrer);
         }

         /// <summary>
         /// POST: /Category/Add
         /// Posts values from the Add razor view
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public async Task<IActionResult> Add(CategoryViewModel viewModel, string returnUrl)
         {
             return await this.ValidateAndSave(CrudOperation.Add, viewModel, returnUrl);
         }

         /// <summary>
         /// GET: /Category/Update/*
         /// Gets the view model used by the Update razor view
         /// </summary>
         public async Task<IActionResult> Update(int id, string urlReferrer = null)
         {
             // return view model
             return await this.GetUpdateViewModelAsync(id, urlReferrer);
         }

         /// <summary>
         /// POST: /Category/Update/#
         /// Posts values from the Update razor view
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public async Task<IActionResult> Update(int id, CategoryViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
                 return await this.ValidateAndSave(CrudOperation.Update, viewModel, returnUrl, id);

             // if we got this far, something failed, redisplay form
             return await this.GetUpdateViewModelAsync(id, returnUrl);
         }

         /// <summary>
         /// GET: /Details/Details/#
         /// Gets the view model used by the Details razor view
         /// </summary>
         public async Task<IActionResult> Details(int id, string urlReferrer)
         {
             // fill the model by primary key
             await this.FillModelByPrimaryKeyAsync(id);

             // set and return the view model
             _categoryViewModel = this.GetViewModel("Details", "Category", urlReferrer, _category, CrudOperation.None);
             return View(_categoryViewModel);
         }

         /// <summary>
         /// GET: /Category/ListCrudRedirect/
         /// Gets the view used by the ListCrudRedirect razor view
         /// </summary>
         public IActionResult ListCrudRedirect()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Category/ListReadOnly/
         /// Gets the view used by the ListReadOnly razor view
         /// </summary>
         public IActionResult ListReadOnly()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Category/ListCrud/
         /// Gets the view model used by the ListCrud razor view
         /// </summary>
         public IActionResult ListCrud()
         {
             // return view
             _categoryViewModel = this.GetViewModel("ListCrud");
             return View();
         }

         /// <summary>
         /// POST: /Category/ListCrud
         /// Posts values from the ListCrud
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> ListCrud(string inputSubmit, string serializedData)
         {
             // deserialize serializedData
             _category = JsonConvert.DeserializeObject<Category>(serializedData);

             // assign a value to the view model's Category (Model) property
             _categoryViewModel.Category = _category;

             if (ModelState.IsValid)
             {
                 CrudOperation operation = CrudOperation.Add;

                 if (inputSubmit == "Update")
                     operation = CrudOperation.Update;

                 try
                 {
                     // add a new record or update an existing record
                     await AddEditCategoryAsync(_categoryViewModel, operation, true);
                 }
                 catch(Exception ex)
                 {
                     ModelState.AddModelError("", ex.Message);
                 }
             }

             return Json(true);
         }

         /// <summary>
         /// GET: /Category/ListSearch/
         /// Gets the view model used by the ListSearch razor view
         /// </summary>
         public IActionResult ListSearch()
         {
             // return view
             _categoryViewModel = this.GetViewModel("ListSearch");
             return View();
         }

         /// <summary>
         /// GET: /Category/ListScrollLoad/
         /// Gets the view used by the ListScrollLoad razor view
         /// </summary>
         public IActionResult ListScrollLoad()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Category/ListInline/
         /// Gets the view model used by the ListInline razor view
         /// </summary>
         public IActionResult ListInline()
         {
             // return view
             _categoryViewModel = this.GetViewModel("ListInline");
             return View();
         }

         /// <summary>
         /// POST: /Category/ListInlineAdd
         /// Posts the values from the ListInlineAdd
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> ListInlineAdd([FromBody]string modelString)
         {
             return await this.ListInlineAddOrUpdate(modelString, CrudOperation.Add);
         }

         /// <summary>
         /// POST: /Category/ListInlineUpdate
         /// Posts the values from the ListInlineUpdate
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> ListInlineUpdate([FromBody]string modelString)
         {
             return await this.ListInlineAddOrUpdate(modelString, CrudOperation.Update);
         }

         /// <summary>
         /// GET: /Category/ListForeach
         /// Gets the view model used by the ListForeach razor view
         /// </summary>
         public async Task<IActionResult> ListForeach(string sidx, string sord, int? page)
         {
             // get the default number of rows to show
             int rows = Functions.GetGridNumberOfRows();

             // the numberOfPagesToShow is used to show the "< First..." link in the razor view
             int numberOfPagesToShow = Functions.GetGridNumberOfPagesToShow();

             // when page is null, set it to 1
             int currentPage = page is null ? 1 : Convert.ToInt32(page);

             // get the total number of records
             int totalRecords = await _categoryBusinessLayer.GetRecordCountAsync();

             // get the index where to start retrieving records from and the total pages
             (int startRowIndex, int totalPages) = Functions.GetStartRowIndexAndTotalPages(currentPage, rows, totalRecords);

             // get records based on filters
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // fields and titles
             string[,] fieldNames = new string[,] {
                 {"CategoryID", "Category ID"},
                 {"CategoryName", "Category Name"},
                 {"Description", "Description"}
             };

             // fields that are not sortable
             List<string> unsortableFields = new();
             unsortableFields.Add("Description");

             // assign values to the view model
             _categoryForEachViewModel.CategoryData = objCategoriesList;
             _categoryForEachViewModel.CategoryFieldNames = fieldNames;
             _categoryForEachViewModel.TotalPages = totalPages;
             _categoryForEachViewModel.CurrentPage = currentPage;
             _categoryForEachViewModel.FieldToSort = String.IsNullOrEmpty(sidx) ? "CategoryID" : sidx;
             _categoryForEachViewModel.FieldSortOrder = String.IsNullOrEmpty(sord) ? "asc" : sord;
             _categoryForEachViewModel.FieldToSortWithOrder = String.IsNullOrEmpty(sidx) ? "CategoryID" : (sidx + " " + sord).Trim();
             _categoryForEachViewModel.NumberOfPagesToShow = numberOfPagesToShow;
             _categoryForEachViewModel.StartPage = Functions.GetPagerStartPage(currentPage, numberOfPagesToShow);
             _categoryForEachViewModel.EndPage = Functions.GetPagerEndPage(_categoryForEachViewModel.StartPage, numberOfPagesToShow, totalPages);
             _categoryForEachViewModel.UnsortableFields = unsortableFields;

             // return the view model
             return View(_categoryForEachViewModel);
         }

         /// <summary>
         /// GET: /Category/ListMultipleDelete/
         /// Gets the view used by the ListMultipleDelete razor view
         /// </summary>
         public IActionResult ListMultipleDelete()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Category/Unbound
         /// Gets the view model used by the Unbound razor view
         /// </summary>
         public IActionResult Unbound()
         {
             // set and return the view model
             _categoryViewModel = this.GetViewModel("Unbound", "Category", "/Home", null, CrudOperation.None);
             return View(_categoryViewModel);
         }

         /// <summary>
         /// POST: /Category/Unbound
         /// Post values fromy the Unbound razor view
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Unbound(CategoryViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 // do something here before redirecting

                 if (Url.IsLocalUrl(returnUrl))
                     return Redirect(returnUrl);
                 else
                     return RedirectToAction("Index", "Home");
             }

             // if we got this far, something failed, redisplay form
             _categoryViewModel = this.GetViewModel("Unbound", "Category", "/Home", null, CrudOperation.None);
             return View(_categoryViewModel);
         }

         #endregion

         #region public methods

         /// <summary>
         /// POST: /Category/Delete/#
         /// Deletes a record based on the id(s)
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> Delete(int id)
         {
             // delete record
             await _categoryBusinessLayer.DeleteAsync(id);
             return Json(true);
         }

         /// <summary>
         /// POST: /Category/DeleteMultiple/ids
         /// Deletes multiple records based on the comma-delimited ids
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> DeleteMultiple(string ids)
         {
             // split ids into a List
             List<Int32> categoryIDList = ids.Split(",").Select(Int32.Parse).ToList();

             // delete multiple records
             await _categoryBusinessLayer.DeleteMultipleAsync(categoryIDList);
             return Json(true);
         }

         #endregion

         #region private methods

         /// <summary>
         /// Gets the view model used by the Add razor view
         /// </summary>
         private IActionResult GetAddViewModel(string returnUrl = null)
         {
             // set the return url to /ListCrudRedirect when the returnUrl paramater is empty
             string url = String.IsNullOrEmpty(returnUrl) ? "/Category/ListCrudRedirect" : returnUrl;

             // set and return the view model
             _categoryViewModel = this.GetViewModel("Add", "Category", url, null, CrudOperation.Add);
             return View(_categoryViewModel);
         }

         /// <summary>
         /// Gets the view model used by the Update razor view
         /// </summary>
         private async Task<IActionResult> GetUpdateViewModelAsync(int id, string urlReferrer)
         {
             // fill the model by primary key
             await this.FillModelByPrimaryKeyAsync(id);

             // set and return the view model
             _categoryViewModel = this.GetViewModel("Update", "Category", urlReferrer, _category, CrudOperation.Update);
             return View(_categoryViewModel);
         }

         /// <summary>
         /// Used when adding a new record or updating an existing record
         /// </summary>
         /// <param name="viewModel">CategoryViewModel</param>
         /// <param name="operation">Operation to Add a new record or Update an existing record</param>
         /// <param name="isForListInlineOrListCrud">Used by the razor Views with ListInline or ListCrud when true</param>
         /// <returns>Task of IActionResult</returns>
         private async Task<IActionResult> AddEditCategoryAsync(CategoryViewModel viewModel, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             Category model = viewModel.Category;

             try
             {
                 // create a new instance of the Category when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    _category = new();
                 else
                    _category = await _categoryBusinessLayer.SelectByPrimaryKeyAsync(model.CategoryID);

                 // assign values to the Category instance
                 _category.CategoryID = model.CategoryID;
                 _category.CategoryName = model.CategoryName;
                 _category.Description = model.Description;

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await _categoryBusinessLayer.InsertAsync(_category);
                 else
                    await _categoryBusinessLayer.UpdateAsync(_category);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }

         /// <summary>
         /// Gets the view model based on the actionName
         /// </summary>
         private CategoryViewModel GetViewModel(string actionName,
         string controllerName = "Category", string returnUrl = null, Category objCategory = null,
         CrudOperation operation = CrudOperation.None)
         {
             // assign values to the view model
             _categoryViewModel.Category = objCategory;
             _categoryViewModel.Operation = operation;
             _categoryViewModel.ViewControllerName = controllerName;
             _categoryViewModel.ViewActionName = actionName;
             _categoryViewModel.ViewReturnUrl = returnUrl;

             // return the view model
             return _categoryViewModel;
         }

         /// <summary>
         /// Validates the Add or Update operation and Saves the data when valid and no errors.
         /// </summary>
         /// <param name="operation">Add or Update only.  CrudOperation.None is not permitted</param>
         /// <param name="categoryViewModel">The View Model</param>
         /// <param name="returnUrl">Url to return to after the operation</param>
         /// <param name="id">Optional.  But required when operation = CrudOperation.Update</param>
         /// <returns>View Model as IActionResult</returns>
         private async Task<IActionResult> ValidateAndSave(CrudOperation operation, CategoryViewModel categoryViewModel, string returnUrl, int? id = null)
         {
             if (operation == CrudOperation.None)
                 throw new ArgumentException("operation = CrudOperation.None is not valid for the ValidateAndSave method.");

             if (ModelState.IsValid)
             {
                 try
                 {
                     // add or update record
                     await this.AddEditCategoryAsync(categoryViewModel, operation);

                     if (Url.IsLocalUrl(returnUrl))
                         return Redirect(returnUrl);
                     else
                         return RedirectToAction("ListCrudRedirect", "Category");
                 }
                 catch (Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             if (operation == CrudOperation.Add)
                 return GetAddViewModel(returnUrl);
             else
                 return await this.GetUpdateViewModelAsync(id.Value, returnUrl);
         }

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sord filtered by the passed parameters
         /// </summary>
         private async Task<List<Category>> GetFilteredDataAsync(int? categoryID, string categoryName, string sidx, string sord, int rows, int startRowIndex, string sortExpression)
         {
             if (categoryID != null || !String.IsNullOrEmpty(categoryName))
                 return await _categoryBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, rows, startRowIndex, sortExpression);

             return await _categoryBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sortExpression);
         }

         /// <summary>
         /// Fills the Category model information by primary key
         /// </summary>
         private async Task FillModelByPrimaryKeyAsync(int categoryID)
         {
             // select a record by primary key(s)
             Category objCategory = await _categoryBusinessLayer.SelectByPrimaryKeyAsync(categoryID);

             // assign values to the model
             _category.CategoryID = objCategory.CategoryID;
             _category.CategoryName = objCategory.CategoryName;
             _category.Description = objCategory.Description;

             // assign values to the model
             _category.CategoryID = objCategory.CategoryID;
             _category.CategoryName = objCategory.CategoryName;
             _category.Description = objCategory.Description;
         }

         /// <summary>
         /// Deserializes the modelString and then Adds a New Record or Updates and existing record.
         /// This method is used by ListInlineAdd or ListInlineUpdate.
         /// </summary>
         private async Task<IActionResult> ListInlineAddOrUpdate(string modelString, CrudOperation operation)
         {
             // deserialize modelString
             _categoryViewModel.Category = JsonConvert.DeserializeObject<Category>("{" + modelString + "}");

             // add new record or update existing record
             await this.AddEditCategoryAsync(_categoryViewModel, operation, true);
             return Json(true);
         }

         /// <summary>
         /// Gets json-formatted data based on the List of Categories for use by the jqgrid.
         /// </summary>
         /// <param name="objCategoriesList">List of Categories</param>
         /// <param name="totalPages">Total number of pages</param>
         /// <param name="page">Current page the grid is on</param>
         /// <param name="totalRecords">Total number of records</param>
         /// <returns>List of Categories in Json format</returns>
         private JsonResult GetJsonData(List<Category> objCategoriesList, int totalPages, int page, int totalRecords)
         {
             // return a null in json for use by the jqgrid
             if (objCategoriesList is null)
                 return Json("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 page,
                 records = totalRecords,
                 rows = (
                     from objCategory in objCategoriesList
                     select new
                     {
                         id = objCategory.CategoryID,
                         cell = new string[] { 
                             objCategory.CategoryID.ToString(),
                             objCategory.CategoryName,
                             objCategory.Description
                         }
                     }).ToArray()
             };

             return Json(jsonData);
         }

         #endregion

         #region methods that return data in json format used by the jqgrid

         /// <summary>
         /// GET: /Category/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> GridData(string sidx, string sord, int page, int rows)
         {
             // get the total number of records
             int totalRecords = await _categoryBusinessLayer.GetRecordCountAsync();

             // get the index where to start retrieving records from and the total number of pages
             (int startRowIndex, int totalPages) = Functions.GetStartRowIndexAndTotalPages(page, rows, totalRecords);

             // get records
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // return json data for use by the jqgrid
             return this.GetJsonData(objCategoriesList, totalPages, page, totalRecords);
         }

         /// <summary>
         /// GET: /Category/GridDataWithFilters
         /// Gets the json needed by the jqgrid for use by the GridDataWithFilters
         /// </summary>
         public async Task<IActionResult> GridDataWithFilters(string _search, string nd, int rows, int page, string sidx, string sord, string filters = "")
         {
             int? categoryID = null;
             string categoryName = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "categoryid")
                         categoryID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "categoryname")
                         categoryName = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (categoryID == -1)
                     categoryID = null;

             }
             // get records based on filters
             int totalRecords = await _categoryBusinessLayer.GetRecordCountDynamicWhereAsync(categoryID, categoryName);

             // get the index where to start retrieving records from and the total pages
             (int startRowIndex, int totalPages) = Functions.GetStartRowIndexAndTotalPages(page, rows, totalRecords);

             // get records based on filters
             List<Category> objCategoriesList = await _categoryBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, rows, startRowIndex, sidx + " " + sord);

             // return json data for use by the jqgrid
             return this.GetJsonData(objCategoriesList, totalPages, page, totalRecords);
         }

         #endregion
     }
}
