using Newtonsoft.Json;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using AdHocApi;
using AdHocApi.BusinessLayer;
using AdHocApi.Models;
using AdHocApi.ViewModels;
using AdHocApi.Domain;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace AdHoc.Controllers
{
     /// <summary>
     /// Works like the Base class for SupplierController class.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the SupplierController class under the Controller folder. **
     /// *************************************************************************************************************
     /// </summary>
     public partial class SupplierController : Controller
     {
         private Supplier _supplier;
         private SupplierViewModel _supplierViewModel;
         private SupplierForeachViewModel _supplierForEachViewModel;
         private readonly ISupplierBusinessLayer _supplierBusinessLayer;

         // constructor
         public SupplierController(Supplier supplier, ISupplierBusinessLayer supplierBusinessLayer, SupplierViewModel supplierViewModel, SupplierForeachViewModel supplierForeachViewModel)
         {
             _supplier = supplier;
             _supplierViewModel = supplierViewModel;
             _supplierForEachViewModel = supplierForeachViewModel;
             _supplierBusinessLayer = supplierBusinessLayer;
         }

         #region actions used by their respective views

         /// <summary>
         /// GET: /Supplier/
         /// Gets the view used by the Index razor view
         /// </summary>
         public IActionResult Index()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Supplier/Add
         /// Gets the view model used by the Add razor view
         /// </summary>
         public IActionResult Add(string urlReferrer = null)
         {
             // return the view model
             return this.GetAddViewModel(urlReferrer);
         }

         /// <summary>
         /// POST: /Supplier/Add
         /// Posts values from the Add razor view
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public async Task<IActionResult> Add(SupplierViewModel viewModel, string returnUrl)
         {
             return await this.ValidateAndSave(CrudOperation.Add, viewModel, returnUrl);
         }

         /// <summary>
         /// GET: /Supplier/Update/*
         /// Gets the view model used by the Update razor view
         /// </summary>
         public async Task<IActionResult> Update(int id, string urlReferrer = null)
         {
             // return view model
             return await this.GetUpdateViewModelAsync(id, urlReferrer);
         }

         /// <summary>
         /// POST: /Supplier/Update/#
         /// Posts values from the Update razor view
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public async Task<IActionResult> Update(int id, SupplierViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
                 return await this.ValidateAndSave(CrudOperation.Update, viewModel, returnUrl, id);

             // if we got this far, something failed, redisplay form
             return await this.GetUpdateViewModelAsync(id, returnUrl);
         }

         /// <summary>
         /// GET: /Details/Details/#
         /// Gets the view model used by the Details razor view
         /// </summary>
         public async Task<IActionResult> Details(int id, string urlReferrer)
         {
             // fill the model by primary key
             await this.FillModelByPrimaryKeyAsync(id);

             // set and return the view model
             _supplierViewModel = this.GetViewModel("Details", "Supplier", urlReferrer, _supplier, CrudOperation.None);
             return View(_supplierViewModel);
         }

         /// <summary>
         /// GET: /Supplier/ListCrudRedirect/
         /// Gets the view used by the ListCrudRedirect razor view
         /// </summary>
         public IActionResult ListCrudRedirect()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Supplier/ListReadOnly/
         /// Gets the view used by the ListReadOnly razor view
         /// </summary>
         public IActionResult ListReadOnly()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Supplier/ListCrud/
         /// Gets the view model used by the ListCrud razor view
         /// </summary>
         public IActionResult ListCrud()
         {
             // return view
             _supplierViewModel = this.GetViewModel("ListCrud");
             return View();
         }

         /// <summary>
         /// POST: /Supplier/ListCrud
         /// Posts values from the ListCrud
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> ListCrud(string inputSubmit, string serializedData)
         {
             // deserialize serializedData
             _supplier = JsonConvert.DeserializeObject<Supplier>(serializedData);

             // assign a value to the view model's Supplier (Model) property
             _supplierViewModel.Supplier = _supplier;

             if (ModelState.IsValid)
             {
                 CrudOperation operation = CrudOperation.Add;

                 if (inputSubmit == "Update")
                     operation = CrudOperation.Update;

                 try
                 {
                     // add a new record or update an existing record
                     await AddEditSupplierAsync(_supplierViewModel, operation, true);
                 }
                 catch(Exception ex)
                 {
                     ModelState.AddModelError("", ex.Message);
                 }
             }

             return Json(true);
         }

         /// <summary>
         /// GET: /Supplier/ListSearch/
         /// Gets the view model used by the ListSearch razor view
         /// </summary>
         public IActionResult ListSearch()
         {
             // return view
             _supplierViewModel = this.GetViewModel("ListSearch");
             return View();
         }

         /// <summary>
         /// GET: /Supplier/ListScrollLoad/
         /// Gets the view used by the ListScrollLoad razor view
         /// </summary>
         public IActionResult ListScrollLoad()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Supplier/ListInline/
         /// Gets the view model used by the ListInline razor view
         /// </summary>
         public IActionResult ListInline()
         {
             // return view
             _supplierViewModel = this.GetViewModel("ListInline");
             return View();
         }

         /// <summary>
         /// POST: /Supplier/ListInlineAdd
         /// Posts the values from the ListInlineAdd
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> ListInlineAdd([FromBody]string modelString)
         {
             return await this.ListInlineAddOrUpdate(modelString, CrudOperation.Add);
         }

         /// <summary>
         /// POST: /Supplier/ListInlineUpdate
         /// Posts the values from the ListInlineUpdate
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> ListInlineUpdate([FromBody]string modelString)
         {
             return await this.ListInlineAddOrUpdate(modelString, CrudOperation.Update);
         }

         /// <summary>
         /// GET: /Supplier/ListForeach
         /// Gets the view model used by the ListForeach razor view
         /// </summary>
         public async Task<IActionResult> ListForeach(string sidx, string sord, int? page)
         {
             // get the default number of rows to show
             int rows = Functions.GetGridNumberOfRows();

             // the numberOfPagesToShow is used to show the "< First..." link in the razor view
             int numberOfPagesToShow = Functions.GetGridNumberOfPagesToShow();

             // when page is null, set it to 1
             int currentPage = page is null ? 1 : Convert.ToInt32(page);

             // get the total number of records
             int totalRecords = await _supplierBusinessLayer.GetRecordCountAsync();

             // get the index where to start retrieving records from and the total pages
             (int startRowIndex, int totalPages) = Functions.GetStartRowIndexAndTotalPages(currentPage, rows, totalRecords);

             // get records based on filters
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // fields and titles
             string[,] fieldNames = new string[,] {
                 {"SupplierID", "Supplier ID"},
                 {"CompanyName", "Company Name"},
                 {"ContactName", "Contact Name"},
                 {"ContactTitle", "Contact Title"},
                 {"Address", "Address"},
                 {"City", "City"},
                 {"Region1", "Region1"},
                 {"PostalCode", "Postal Code"},
                 {"Country", "Country"},
                 {"Phone", "Phone"},
                 {"Fax", "Fax"},
                 {"HomePage", "Home Page"}
             };

             // fields that are not sortable
             List<string> unsortableFields = new();
             unsortableFields.Add("HomePage");

             // assign values to the view model
             _supplierForEachViewModel.SupplierData = objSuppliersList;
             _supplierForEachViewModel.SupplierFieldNames = fieldNames;
             _supplierForEachViewModel.TotalPages = totalPages;
             _supplierForEachViewModel.CurrentPage = currentPage;
             _supplierForEachViewModel.FieldToSort = String.IsNullOrEmpty(sidx) ? "SupplierID" : sidx;
             _supplierForEachViewModel.FieldSortOrder = String.IsNullOrEmpty(sord) ? "asc" : sord;
             _supplierForEachViewModel.FieldToSortWithOrder = String.IsNullOrEmpty(sidx) ? "SupplierID" : (sidx + " " + sord).Trim();
             _supplierForEachViewModel.NumberOfPagesToShow = numberOfPagesToShow;
             _supplierForEachViewModel.StartPage = Functions.GetPagerStartPage(currentPage, numberOfPagesToShow);
             _supplierForEachViewModel.EndPage = Functions.GetPagerEndPage(_supplierForEachViewModel.StartPage, numberOfPagesToShow, totalPages);
             _supplierForEachViewModel.UnsortableFields = unsortableFields;

             // return the view model
             return View(_supplierForEachViewModel);
         }

         /// <summary>
         /// GET: /Supplier/ListMultipleDelete/
         /// Gets the view used by the ListMultipleDelete razor view
         /// </summary>
         public IActionResult ListMultipleDelete()
         {
             // return view
             return View();
         }

         /// <summary>
         /// GET: /Supplier/Unbound
         /// Gets the view model used by the Unbound razor view
         /// </summary>
         public IActionResult Unbound()
         {
             // set and return the view model
             _supplierViewModel = this.GetViewModel("Unbound", "Supplier", "/Home", null, CrudOperation.None);
             return View(_supplierViewModel);
         }

         /// <summary>
         /// POST: /Supplier/Unbound
         /// Post values fromy the Unbound razor view
         /// </summary>
         [HttpPost]
         [ValidateAntiForgeryToken]
         public IActionResult Unbound(SupplierViewModel viewModel, string returnUrl)
         {
             if (ModelState.IsValid)
             {
                 // do something here before redirecting

                 if (Url.IsLocalUrl(returnUrl))
                     return Redirect(returnUrl);
                 else
                     return RedirectToAction("Index", "Home");
             }

             // if we got this far, something failed, redisplay form
             _supplierViewModel = this.GetViewModel("Unbound", "Supplier", "/Home", null, CrudOperation.None);
             return View(_supplierViewModel);
         }

         #endregion

         #region public methods

         /// <summary>
         /// POST: /Supplier/Delete/#
         /// Deletes a record based on the id(s)
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> Delete(int id)
         {
             // delete record
             await _supplierBusinessLayer.DeleteAsync(id);
             return Json(true);
         }

         /// <summary>
         /// POST: /Supplier/DeleteMultiple/ids
         /// Deletes multiple records based on the comma-delimited ids
         /// </summary>
         [HttpPost]
         public async Task<IActionResult> DeleteMultiple(string ids)
         {
             // split ids into a List
             List<Int32> supplierIDList = ids.Split(",").Select(Int32.Parse).ToList();

             // delete multiple records
             await _supplierBusinessLayer.DeleteMultipleAsync(supplierIDList);
             return Json(true);
         }

         #endregion

         #region private methods

         /// <summary>
         /// Gets the view model used by the Add razor view
         /// </summary>
         private IActionResult GetAddViewModel(string returnUrl = null)
         {
             // set the return url to /ListCrudRedirect when the returnUrl paramater is empty
             string url = String.IsNullOrEmpty(returnUrl) ? "/Supplier/ListCrudRedirect" : returnUrl;

             // set and return the view model
             _supplierViewModel = this.GetViewModel("Add", "Supplier", url, null, CrudOperation.Add);
             return View(_supplierViewModel);
         }

         /// <summary>
         /// Gets the view model used by the Update razor view
         /// </summary>
         private async Task<IActionResult> GetUpdateViewModelAsync(int id, string urlReferrer)
         {
             // fill the model by primary key
             await this.FillModelByPrimaryKeyAsync(id);

             // set and return the view model
             _supplierViewModel = this.GetViewModel("Update", "Supplier", urlReferrer, _supplier, CrudOperation.Update);
             return View(_supplierViewModel);
         }

         /// <summary>
         /// Used when adding a new record or updating an existing record
         /// </summary>
         /// <param name="viewModel">SupplierViewModel</param>
         /// <param name="operation">Operation to Add a new record or Update an existing record</param>
         /// <param name="isForListInlineOrListCrud">Used by the razor Views with ListInline or ListCrud when true</param>
         /// <returns>Task of IActionResult</returns>
         private async Task<IActionResult> AddEditSupplierAsync(SupplierViewModel viewModel, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             Supplier model = viewModel.Supplier;

             try
             {
                 // create a new instance of the Supplier when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    _supplier = new();
                 else
                    _supplier = await _supplierBusinessLayer.SelectByPrimaryKeyAsync(model.SupplierID);

                 // assign values to the Supplier instance
                 _supplier.SupplierID = model.SupplierID;
                 _supplier.CompanyName = model.CompanyName;
                 _supplier.ContactName = model.ContactName;
                 _supplier.ContactTitle = model.ContactTitle;
                 _supplier.Address = model.Address;
                 _supplier.City = model.City;
                 _supplier.Region1 = model.Region1;
                 _supplier.PostalCode = model.PostalCode;
                 _supplier.Country = model.Country;
                 _supplier.Phone = model.Phone;
                 _supplier.Fax = model.Fax;
                 _supplier.HomePage = model.HomePage;

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await _supplierBusinessLayer.InsertAsync(_supplier);
                 else
                    await _supplierBusinessLayer.UpdateAsync(_supplier);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message);
             }
         }

         /// <summary>
         /// Gets the view model based on the actionName
         /// </summary>
         private SupplierViewModel GetViewModel(string actionName,
         string controllerName = "Supplier", string returnUrl = null, Supplier objSupplier = null,
         CrudOperation operation = CrudOperation.None)
         {
             // assign values to the view model
             _supplierViewModel.Supplier = objSupplier;
             _supplierViewModel.Operation = operation;
             _supplierViewModel.ViewControllerName = controllerName;
             _supplierViewModel.ViewActionName = actionName;
             _supplierViewModel.ViewReturnUrl = returnUrl;

             // return the view model
             return _supplierViewModel;
         }

         /// <summary>
         /// Validates the Add or Update operation and Saves the data when valid and no errors.
         /// </summary>
         /// <param name="operation">Add or Update only.  CrudOperation.None is not permitted</param>
         /// <param name="supplierViewModel">The View Model</param>
         /// <param name="returnUrl">Url to return to after the operation</param>
         /// <param name="id">Optional.  But required when operation = CrudOperation.Update</param>
         /// <returns>View Model as IActionResult</returns>
         private async Task<IActionResult> ValidateAndSave(CrudOperation operation, SupplierViewModel supplierViewModel, string returnUrl, int? id = null)
         {
             if (operation == CrudOperation.None)
                 throw new ArgumentException("operation = CrudOperation.None is not valid for the ValidateAndSave method.");

             if (ModelState.IsValid)
             {
                 try
                 {
                     // add or update record
                     await this.AddEditSupplierAsync(supplierViewModel, operation);

                     if (Url.IsLocalUrl(returnUrl))
                         return Redirect(returnUrl);
                     else
                         return RedirectToAction("ListCrudRedirect", "Supplier");
                 }
                 catch (Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             if (operation == CrudOperation.Add)
                 return GetAddViewModel(returnUrl);
             else
                 return await this.GetUpdateViewModelAsync(id.Value, returnUrl);
         }

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sord filtered by the passed parameters
         /// </summary>
         private async Task<List<Supplier>> GetFilteredDataAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sidx, string sord, int rows, int startRowIndex, string sortExpression)
         {
             if (supplierID != null || !String.IsNullOrEmpty(companyName) || !String.IsNullOrEmpty(contactName) || !String.IsNullOrEmpty(contactTitle) || !String.IsNullOrEmpty(address) || !String.IsNullOrEmpty(city) || !String.IsNullOrEmpty(region1) || !String.IsNullOrEmpty(postalCode) || !String.IsNullOrEmpty(country) || !String.IsNullOrEmpty(phone) || !String.IsNullOrEmpty(fax))
                 return await _supplierBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sortExpression);

             return await _supplierBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sortExpression);
         }

         /// <summary>
         /// Fills the Supplier model information by primary key
         /// </summary>
         private async Task FillModelByPrimaryKeyAsync(int supplierID)
         {
             // select a record by primary key(s)
             Supplier objSupplier = await _supplierBusinessLayer.SelectByPrimaryKeyAsync(supplierID);

             // assign values to the model
             _supplier.SupplierID = objSupplier.SupplierID;
             _supplier.CompanyName = objSupplier.CompanyName;
             _supplier.ContactName = objSupplier.ContactName;
             _supplier.ContactTitle = objSupplier.ContactTitle;
             _supplier.Address = objSupplier.Address;
             _supplier.City = objSupplier.City;
             _supplier.Region1 = objSupplier.Region1;
             _supplier.PostalCode = objSupplier.PostalCode;
             _supplier.Country = objSupplier.Country;
             _supplier.Phone = objSupplier.Phone;
             _supplier.Fax = objSupplier.Fax;
             _supplier.HomePage = objSupplier.HomePage;

             // assign values to the model
             _supplier.SupplierID = objSupplier.SupplierID;
             _supplier.CompanyName = objSupplier.CompanyName;
             _supplier.ContactName = objSupplier.ContactName;
             _supplier.ContactTitle = objSupplier.ContactTitle;
             _supplier.Address = objSupplier.Address;
             _supplier.City = objSupplier.City;
             _supplier.Region1 = objSupplier.Region1;
             _supplier.PostalCode = objSupplier.PostalCode;
             _supplier.Country = objSupplier.Country;
             _supplier.Phone = objSupplier.Phone;
             _supplier.Fax = objSupplier.Fax;
             _supplier.HomePage = objSupplier.HomePage;
         }

         /// <summary>
         /// Deserializes the modelString and then Adds a New Record or Updates and existing record.
         /// This method is used by ListInlineAdd or ListInlineUpdate.
         /// </summary>
         private async Task<IActionResult> ListInlineAddOrUpdate(string modelString, CrudOperation operation)
         {
             // deserialize modelString
             _supplierViewModel.Supplier = JsonConvert.DeserializeObject<Supplier>("{" + modelString + "}");

             // add new record or update existing record
             await this.AddEditSupplierAsync(_supplierViewModel, operation, true);
             return Json(true);
         }

         /// <summary>
         /// Gets json-formatted data based on the List of Suppliers for use by the jqgrid.
         /// </summary>
         /// <param name="objSuppliersList">List of Suppliers</param>
         /// <param name="totalPages">Total number of pages</param>
         /// <param name="page">Current page the grid is on</param>
         /// <param name="totalRecords">Total number of records</param>
         /// <returns>List of Suppliers in Json format</returns>
         private JsonResult GetJsonData(List<Supplier> objSuppliersList, int totalPages, int page, int totalRecords)
         {
             // return a null in json for use by the jqgrid
             if (objSuppliersList is null)
                 return Json("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 page,
                 records = totalRecords,
                 rows = (
                     from objSupplier in objSuppliersList
                     select new
                     {
                         id = objSupplier.SupplierID,
                         cell = new string[] { 
                             objSupplier.SupplierID.ToString(),
                             objSupplier.CompanyName,
                             objSupplier.ContactName,
                             objSupplier.ContactTitle,
                             objSupplier.Address,
                             objSupplier.City,
                             objSupplier.Region1,
                             objSupplier.PostalCode,
                             objSupplier.Country,
                             objSupplier.Phone,
                             objSupplier.Fax,
                             objSupplier.HomePage
                         }
                     }).ToArray()
             };

             return Json(jsonData);
         }

         #endregion

         #region methods that return data in json format used by the jqgrid

         /// <summary>
         /// GET: /Supplier/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> GridData(string sidx, string sord, int page, int rows)
         {
             // get the total number of records
             int totalRecords = await _supplierBusinessLayer.GetRecordCountAsync();

             // get the index where to start retrieving records from and the total number of pages
             (int startRowIndex, int totalPages) = Functions.GetStartRowIndexAndTotalPages(page, rows, totalRecords);

             // get records
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // return json data for use by the jqgrid
             return this.GetJsonData(objSuppliersList, totalPages, page, totalRecords);
         }

         /// <summary>
         /// GET: /Supplier/GridDataWithFilters
         /// Gets the json needed by the jqgrid for use by the GridDataWithFilters
         /// </summary>
         public async Task<IActionResult> GridDataWithFilters(string _search, string nd, int rows, int page, string sidx, string sord, string filters = "")
         {
             int? supplierID = null;
             string companyName = String.Empty;
             string contactName = String.Empty;
             string contactTitle = String.Empty;
             string address = String.Empty;
             string city = String.Empty;
             string region1 = String.Empty;
             string postalCode = String.Empty;
             string country = String.Empty;
             string phone = String.Empty;
             string fax = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "companyname")
                         companyName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contactname")
                         contactName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contacttitle")
                         contactTitle = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "address")
                         address = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "city")
                         city = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "region1")
                         region1 = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "postalcode")
                         postalCode = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "country")
                         country = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "phone")
                         phone = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "fax")
                         fax = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (supplierID == -1)
                     supplierID = null;

             }
             // get records based on filters
             int totalRecords = await _supplierBusinessLayer.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

             // get the index where to start retrieving records from and the total pages
             (int startRowIndex, int totalPages) = Functions.GetStartRowIndexAndTotalPages(page, rows, totalRecords);

             // get records based on filters
             List<Supplier> objSuppliersList = await _supplierBusinessLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sidx + " " + sord);

             // return json data for use by the jqgrid
             return this.GetJsonData(objSuppliersList, totalPages, page, totalRecords);
         }

         #endregion
     }
}
