using AdHocApi.DataRepository;
using AdHocApi.Models;
using System;
using System.Collections.Generic;
using System.Data;
using System.Threading.Tasks;

namespace AdHocApi.BusinessLayer
{
     /// <summary>
     /// Works like the Base class for CategoryBusinessLayer class.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the CategoryBusinessLayer class under the BusinessLayer folder. **
     /// *************************************************************************************************************
     /// 
     /// </summary>
     public partial class CategoryBusinessLayer : ICategoryBusinessLayer
     {
         private readonly ICategoryRepository _categoryRepository;

         /// <summary>
         /// constructor
         /// </summary>
         public CategoryBusinessLayer(ICategoryRepository categoryRepository) =>
             _categoryRepository = categoryRepository;

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public async Task<Category> SelectByPrimaryKeyAsync(int categoryID)
         {
             DataTable dt = await _categoryRepository.SelectByPrimaryKeyAsync(categoryID);

             // create Category
             if (dt is not null && dt.Rows.Count > 0)
                 return this.CreateCategoryFromDataRow(dt.Rows[0]);

             return null;
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         public async Task<int> GetRecordCountAsync()
         {
             return await _categoryRepository.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         public async Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName)
         {
             return await _categoryRepository.GetRecordCountDynamicWhereAsync(categoryID, categoryName);
         }

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sortByExpression.
         /// </summary>
         public async Task<List<Category>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             DataTable dt = await _categoryRepository.SelectSkipAndTakeAsync(sortByExpression, startRowIndex, rows);
             return this.GetListOfCategory(dt);
         }

         /// <summary>
         /// Selects records as a collection (List) of Category sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public async Task<List<Category>> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             DataTable dt = await _categoryRepository.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, sortByExpression, startRowIndex, rows);
             return this.GetListOfCategory(dt);
         }

         /// <summary>
         /// Selects all records as a collection (List) of Category
         /// </summary>
         public async Task<List<Category>> SelectAllAsync()
         {
             DataTable dt = await _categoryRepository.SelectAllAsync();
             return this.GetListOfCategory(dt);
         }

         /// <summary>
         /// Selects all records as a collection (List) of Category sorted by the sort expression
         /// </summary>
         public async Task<List<Category>> SelectAllAsync(string sortByExpression)
         {
             DataTable dt = await _categoryRepository.SelectAllAsync();
             List<Category> objCategoriesList = this.GetListOfCategory(dt);
             return await this.SortByExpressionAsync(objCategoriesList, sortByExpression);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Category.
         /// </summary>
         public async Task<List<Category>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName)
         {
             DataTable dt = await _categoryRepository.SelectAllDynamicWhereAsync(categoryID, categoryName);
             return this.GetListOfCategory(dt);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Category sorted by the sort expression.
         /// </summary>
         public async Task<List<Category>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression)
         {
             DataTable dt = await _categoryRepository.SelectAllDynamicWhereAsync(categoryID, categoryName);
             List<Category> objCategoriesList = this.GetListOfCategory(dt);
             return await this.SortByExpressionAsync(objCategoriesList, sortByExpression);
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public async Task<List<Category>> SelectCategoryDropDownListDataAsync()
         {
             List<Category> objCategoriesList = null;
             DataTable dt = await _categoryRepository.SelectCategoryDropDownListDataAsync();

             // build the list of Categories
             if (dt is not null && dt.Rows.Count > 0)
             {
                 objCategoriesList = new();

                 foreach (DataRow dr in dt.Rows)
                 {
                     Category objCategory = new();
                     objCategory.CategoryID = (int)dr["CategoryID"];

                     if(String.IsNullOrEmpty(dr["CategoryName"].ToString()))
                         objCategory.CategoryName = String.Empty;
                     else
                         objCategory.CategoryName = (string)(dr["CategoryName"]);

                     objCategoriesList.Add(objCategory);
                 }
             }

             return objCategoriesList;
         }

         /// <summary>
         /// Sorts the List<Category >by sort expression
         /// </summary>
         public async Task<List<Category>> SortByExpressionAsync(List<Category> objCategoriesList, string sortExpression)
         {
             bool isSortDescending = sortExpression.ToLower().Contains(" desc");

             if (isSortDescending)
             {
                 sortExpression = sortExpression.Replace(" DESC", "");
                 sortExpression = sortExpression.Replace(" desc", "");
             }
             else
             {
                 sortExpression = sortExpression.Replace(" ASC", "");
                 sortExpression = sortExpression.Replace(" asc", "");
             }

             switch (sortExpression)
             {
                 case "CategoryID":
                     await Task.Run(() => objCategoriesList.Sort(AdHocApi.BusinessLayer.CategoryBusinessLayer.ByCategoryID));
                     break;
                 case "CategoryName":
                     await Task.Run(() => objCategoriesList.Sort(AdHocApi.BusinessLayer.CategoryBusinessLayer.ByCategoryName));
                     break;
                 case "Description":
                     await Task.Run(() => objCategoriesList.Sort(AdHocApi.BusinessLayer.CategoryBusinessLayer.ByDescription));
                     break;
                 default:
                     break;
             }

             if (isSortDescending)
                 objCategoriesList.Reverse();

             return objCategoriesList;
         }

         /// <summary>
         /// Inserts a new record
         /// </summary>
         public async Task<int> InsertAsync(Category objCategory)
         {
             return await _categoryRepository.InsertAsync(objCategory);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         public async Task UpdateAsync(Category objCategory) =>
             await _categoryRepository.UpdateAsync(objCategory);

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public async Task DeleteAsync(int categoryID) =>
             await _categoryRepository.DeleteAsync(categoryID);

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public async Task DeleteMultipleAsync(List<Int32> categoryIDList)
         {
             foreach (var id in categoryIDList)
             {
                 await _categoryRepository.DeleteAsync(id);
             }
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, CategoryID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "CategoryID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Gets a List of Categories based on the sql script.
         /// </summary>
         /// <param name="sqlParamList">List of SqlParameters</param>
         /// <param name="sql">The SQL script</param>
         /// <returns>List of Categories</returns>
         private List<Category> GetListOfCategory (DataTable dt)
         {
             List<Category> objCategoriesList = null;

             // build the list of Categories
             if (dt != null && dt.Rows.Count > 0)
             {
                 objCategoriesList = new List<Category>();

                 foreach (DataRow dr in dt.Rows)
                 {
                     Category objCategory = this.CreateCategoryFromDataRow(dr);
                     objCategoriesList.Add(objCategory);
                 }
             }

             return objCategoriesList;
         }

         /// <summary>
         /// Creates a Category object from Data Row
         /// </summary>
         private Category CreateCategoryFromDataRow(DataRow dr)
         {
             // instantiate the Category model
             Category objCategory = new();

             // assign values to the model
             objCategory.CategoryID = (int)dr["CategoryID"];
             objCategory.CategoryName = dr["CategoryName"].ToString();

             if (dr["Description"] != System.DBNull.Value)
                 objCategory.Description = dr["Description"].ToString();
             else
                 objCategory.Description = null;

             return objCategory;
         }

         /// <summary>
         /// Compares CategoryID used for sorting
         /// </summary>
         public static Comparison<Category> ByCategoryID = delegate(Category x, Category y)
         {
             return x.CategoryID.CompareTo(y.CategoryID);
         };

         /// <summary>
         /// Compares CategoryName used for sorting
         /// </summary>
         public static Comparison<Category> ByCategoryName = delegate(Category x, Category y)
         {
             string value1 = x.CategoryName ?? String.Empty;
             string value2 = y.CategoryName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Description used for sorting
         /// </summary>
         public static Comparison<Category> ByDescription = delegate(Category x, Category y)
         {
             string value1 = x.Description ?? String.Empty;
             string value2 = y.Description ?? String.Empty;
             return value1.CompareTo(value2);
         };

     }
}
