using System;
using AdHocApi.Models;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace AdHocApi.BusinessLayer
{
     /// <summary>
     /// Works like the Base interface for ISupplierBusinessLayer interface.
     /// **************************** Do not make changes to this interface ****************************
     /// ** Put your additional code in the ISupplierBusinessLayer interface under the BusinessLayer folder. **
     /// *******************************************************************************************
     /// </summary>
     public partial interface ISupplierBusinessLayer
     {

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public Task<Supplier> SelectByPrimaryKeyAsync(int supplierID);

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         public Task<int> GetRecordCountAsync();

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         public Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax);

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sortByExpression.
         /// </summary>
         public Task<List<Supplier>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression);

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public Task<List<Supplier>> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, int rows, int startRowIndex, string sortByExpression);

         /// <summary>
         /// Selects all records as a collection (List) of Supplier
         /// </summary>
         public Task<List<Supplier>> SelectAllAsync();

         /// <summary>
         /// Selects all records as a collection (List) of Supplier sorted by the sort expression
         /// </summary>
         public Task<List<Supplier>> SelectAllAsync(string sortByExpression);

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Supplier.
         /// </summary>
         public Task<List<Supplier>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax);

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Supplier sorted by the sort expression.
         /// </summary>
         public Task<List<Supplier>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression);

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public Task<List<Supplier>> SelectSupplierDropDownListDataAsync();

         /// <summary>
         /// Sorts the List<Supplier >by sort expression
         /// </summary>
         public Task<List<Supplier>> SortByExpressionAsync(List<Supplier> objSuppliersList, string sortExpression);

         /// <summary>
         /// Inserts a new record
         /// </summary>
         public Task<int> InsertAsync(Supplier objSupplier);

         /// <summary>
         /// Updates a record
         /// </summary>
         public Task UpdateAsync(Supplier objSupplier);

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public Task DeleteAsync(int supplierID);

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public Task DeleteMultipleAsync(List<Int32> supplierIDList);
     }
}
