using AdHocApi.DataRepository;
using AdHocApi.Models;
using System;
using System.Collections.Generic;
using System.Data;
using System.Threading.Tasks;

namespace AdHocApi.BusinessLayer
{
     /// <summary>
     /// Works like the Base class for SupplierBusinessLayer class.
     /// ************************************* Do not make changes to this class *************************************
     /// ** Put your additional code in the SupplierBusinessLayer class under the BusinessLayer folder. **
     /// *************************************************************************************************************
     /// 
     /// </summary>
     public partial class SupplierBusinessLayer : ISupplierBusinessLayer
     {
         private readonly ISupplierRepository _supplierRepository;

         /// <summary>
         /// constructor
         /// </summary>
         public SupplierBusinessLayer(ISupplierRepository supplierRepository) =>
             _supplierRepository = supplierRepository;

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public async Task<Supplier> SelectByPrimaryKeyAsync(int supplierID)
         {
             DataTable dt = await _supplierRepository.SelectByPrimaryKeyAsync(supplierID);

             // create Supplier
             if (dt is not null && dt.Rows.Count > 0)
                 return this.CreateSupplierFromDataRow(dt.Rows[0]);

             return null;
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         public async Task<int> GetRecordCountAsync()
         {
             return await _supplierRepository.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         public async Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             return await _supplierRepository.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
         }

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sortByExpression.
         /// </summary>
         public async Task<List<Supplier>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             DataTable dt = await _supplierRepository.SelectSkipAndTakeAsync(sortByExpression, startRowIndex, rows);
             return this.GetListOfSupplier(dt);
         }

         /// <summary>
         /// Selects records as a collection (List) of Supplier sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public async Task<List<Supplier>> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = this.GetSortExpression(sortByExpression);
             DataTable dt = await _supplierRepository.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, sortByExpression, startRowIndex, rows);
             return this.GetListOfSupplier(dt);
         }

         /// <summary>
         /// Selects all records as a collection (List) of Supplier
         /// </summary>
         public async Task<List<Supplier>> SelectAllAsync()
         {
             DataTable dt = await _supplierRepository.SelectAllAsync();
             return this.GetListOfSupplier(dt);
         }

         /// <summary>
         /// Selects all records as a collection (List) of Supplier sorted by the sort expression
         /// </summary>
         public async Task<List<Supplier>> SelectAllAsync(string sortByExpression)
         {
             DataTable dt = await _supplierRepository.SelectAllAsync();
             List<Supplier> objSuppliersList = this.GetListOfSupplier(dt);
             return await this.SortByExpressionAsync(objSuppliersList, sortByExpression);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Supplier.
         /// </summary>
         public async Task<List<Supplier>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             DataTable dt = await _supplierRepository.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
             return this.GetListOfSupplier(dt);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Supplier sorted by the sort expression.
         /// </summary>
         public async Task<List<Supplier>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression)
         {
             DataTable dt = await _supplierRepository.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
             List<Supplier> objSuppliersList = this.GetListOfSupplier(dt);
             return await this.SortByExpressionAsync(objSuppliersList, sortByExpression);
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public async Task<List<Supplier>> SelectSupplierDropDownListDataAsync()
         {
             List<Supplier> objSuppliersList = null;
             DataTable dt = await _supplierRepository.SelectSupplierDropDownListDataAsync();

             // build the list of Suppliers
             if (dt is not null && dt.Rows.Count > 0)
             {
                 objSuppliersList = new();

                 foreach (DataRow dr in dt.Rows)
                 {
                     Supplier objSupplier = new();
                     objSupplier.SupplierID = (int)dr["SupplierID"];

                     if(String.IsNullOrEmpty(dr["CompanyName"].ToString()))
                         objSupplier.CompanyName = String.Empty;
                     else
                         objSupplier.CompanyName = (string)(dr["CompanyName"]);

                     objSuppliersList.Add(objSupplier);
                 }
             }

             return objSuppliersList;
         }

         /// <summary>
         /// Sorts the List<Supplier >by sort expression
         /// </summary>
         public async Task<List<Supplier>> SortByExpressionAsync(List<Supplier> objSuppliersList, string sortExpression)
         {
             bool isSortDescending = sortExpression.ToLower().Contains(" desc");

             if (isSortDescending)
             {
                 sortExpression = sortExpression.Replace(" DESC", "");
                 sortExpression = sortExpression.Replace(" desc", "");
             }
             else
             {
                 sortExpression = sortExpression.Replace(" ASC", "");
                 sortExpression = sortExpression.Replace(" asc", "");
             }

             switch (sortExpression)
             {
                 case "SupplierID":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.BySupplierID));
                     break;
                 case "CompanyName":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByCompanyName));
                     break;
                 case "ContactName":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByContactName));
                     break;
                 case "ContactTitle":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByContactTitle));
                     break;
                 case "Address":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByAddress));
                     break;
                 case "City":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByCity));
                     break;
                 case "Region1":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByRegion1));
                     break;
                 case "PostalCode":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByPostalCode));
                     break;
                 case "Country":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByCountry));
                     break;
                 case "Phone":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByPhone));
                     break;
                 case "Fax":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByFax));
                     break;
                 case "HomePage":
                     await Task.Run(() => objSuppliersList.Sort(AdHocApi.BusinessLayer.SupplierBusinessLayer.ByHomePage));
                     break;
                 default:
                     break;
             }

             if (isSortDescending)
                 objSuppliersList.Reverse();

             return objSuppliersList;
         }

         /// <summary>
         /// Inserts a new record
         /// </summary>
         public async Task<int> InsertAsync(Supplier objSupplier)
         {
             return await _supplierRepository.InsertAsync(objSupplier);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         public async Task UpdateAsync(Supplier objSupplier) =>
             await _supplierRepository.UpdateAsync(objSupplier);

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public async Task DeleteAsync(int supplierID) =>
             await _supplierRepository.DeleteAsync(supplierID);

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public async Task DeleteMultipleAsync(List<Int32> supplierIDList)
         {
             foreach (var id in supplierIDList)
             {
                 await _supplierRepository.DeleteAsync(id);
             }
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, SupplierID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "SupplierID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Gets a List of Suppliers based on the sql script.
         /// </summary>
         /// <param name="sqlParamList">List of SqlParameters</param>
         /// <param name="sql">The SQL script</param>
         /// <returns>List of Suppliers</returns>
         private List<Supplier> GetListOfSupplier (DataTable dt)
         {
             List<Supplier> objSuppliersList = null;

             // build the list of Suppliers
             if (dt != null && dt.Rows.Count > 0)
             {
                 objSuppliersList = new List<Supplier>();

                 foreach (DataRow dr in dt.Rows)
                 {
                     Supplier objSupplier = this.CreateSupplierFromDataRow(dr);
                     objSuppliersList.Add(objSupplier);
                 }
             }

             return objSuppliersList;
         }

         /// <summary>
         /// Creates a Supplier object from Data Row
         /// </summary>
         private Supplier CreateSupplierFromDataRow(DataRow dr)
         {
             // instantiate the Supplier model
             Supplier objSupplier = new();

             // assign values to the model
             objSupplier.SupplierID = (int)dr["SupplierID"];
             objSupplier.CompanyName = dr["CompanyName"].ToString();

             if (dr["ContactName"] != System.DBNull.Value)
                 objSupplier.ContactName = dr["ContactName"].ToString();
             else
                 objSupplier.ContactName = null;

             if (dr["ContactTitle"] != System.DBNull.Value)
                 objSupplier.ContactTitle = dr["ContactTitle"].ToString();
             else
                 objSupplier.ContactTitle = null;

             if (dr["Address"] != System.DBNull.Value)
                 objSupplier.Address = dr["Address"].ToString();
             else
                 objSupplier.Address = null;

             if (dr["City"] != System.DBNull.Value)
                 objSupplier.City = dr["City"].ToString();
             else
                 objSupplier.City = null;

             if (dr["Region1"] != System.DBNull.Value)
                 objSupplier.Region1 = dr["Region1"].ToString();
             else
                 objSupplier.Region1 = null;

             if (dr["PostalCode"] != System.DBNull.Value)
                 objSupplier.PostalCode = dr["PostalCode"].ToString();
             else
                 objSupplier.PostalCode = null;

             if (dr["Country"] != System.DBNull.Value)
                 objSupplier.Country = dr["Country"].ToString();
             else
                 objSupplier.Country = null;

             if (dr["Phone"] != System.DBNull.Value)
                 objSupplier.Phone = dr["Phone"].ToString();
             else
                 objSupplier.Phone = null;

             if (dr["Fax"] != System.DBNull.Value)
                 objSupplier.Fax = dr["Fax"].ToString();
             else
                 objSupplier.Fax = null;

             if (dr["HomePage"] != System.DBNull.Value)
                 objSupplier.HomePage = dr["HomePage"].ToString();
             else
                 objSupplier.HomePage = null;

             return objSupplier;
         }

         /// <summary>
         /// Compares SupplierID used for sorting
         /// </summary>
         public static Comparison<Supplier> BySupplierID = delegate(Supplier x, Supplier y)
         {
             return x.SupplierID.CompareTo(y.SupplierID);
         };

         /// <summary>
         /// Compares CompanyName used for sorting
         /// </summary>
         public static Comparison<Supplier> ByCompanyName = delegate(Supplier x, Supplier y)
         {
             string value1 = x.CompanyName ?? String.Empty;
             string value2 = y.CompanyName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares ContactName used for sorting
         /// </summary>
         public static Comparison<Supplier> ByContactName = delegate(Supplier x, Supplier y)
         {
             string value1 = x.ContactName ?? String.Empty;
             string value2 = y.ContactName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares ContactTitle used for sorting
         /// </summary>
         public static Comparison<Supplier> ByContactTitle = delegate(Supplier x, Supplier y)
         {
             string value1 = x.ContactTitle ?? String.Empty;
             string value2 = y.ContactTitle ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Address used for sorting
         /// </summary>
         public static Comparison<Supplier> ByAddress = delegate(Supplier x, Supplier y)
         {
             string value1 = x.Address ?? String.Empty;
             string value2 = y.Address ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares City used for sorting
         /// </summary>
         public static Comparison<Supplier> ByCity = delegate(Supplier x, Supplier y)
         {
             string value1 = x.City ?? String.Empty;
             string value2 = y.City ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Region1 used for sorting
         /// </summary>
         public static Comparison<Supplier> ByRegion1 = delegate(Supplier x, Supplier y)
         {
             string value1 = x.Region1 ?? String.Empty;
             string value2 = y.Region1 ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares PostalCode used for sorting
         /// </summary>
         public static Comparison<Supplier> ByPostalCode = delegate(Supplier x, Supplier y)
         {
             string value1 = x.PostalCode ?? String.Empty;
             string value2 = y.PostalCode ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Country used for sorting
         /// </summary>
         public static Comparison<Supplier> ByCountry = delegate(Supplier x, Supplier y)
         {
             string value1 = x.Country ?? String.Empty;
             string value2 = y.Country ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Phone used for sorting
         /// </summary>
         public static Comparison<Supplier> ByPhone = delegate(Supplier x, Supplier y)
         {
             string value1 = x.Phone ?? String.Empty;
             string value2 = y.Phone ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Fax used for sorting
         /// </summary>
         public static Comparison<Supplier> ByFax = delegate(Supplier x, Supplier y)
         {
             string value1 = x.Fax ?? String.Empty;
             string value2 = y.Fax ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares HomePage used for sorting
         /// </summary>
         public static Comparison<Supplier> ByHomePage = delegate(Supplier x, Supplier y)
         {
             string value1 = x.HomePage ?? String.Empty;
             string value2 = y.HomePage ?? String.Empty;
             return value1.CompareTo(value2);
         };

     }
}
