using AdHocApi.Models;
using System;
using System.Data;
using System.Threading.Tasks;

namespace AdHocApi.DataRepository
{
     /// <summary>
     /// Works like the Base interface for ICategoryRepository interface.
     /// **************************** Do not make changes to this interface ****************************
     /// ** Put your additional code in the ICategoryRepository interface under the DataRepository folder. **
     /// ***********************************************************************************************
     /// </summary>
     public partial interface ICategoryRepository
     {
         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         internal Task<DataTable> SelectByPrimaryKeyAsync(int categoryID);

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         internal Task<int> GetRecordCountAsync();

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         internal Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName);

         /// <summary>
         /// Selects Categories table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         internal Task<DataTable> SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows);

         /// <summary>
         /// Selects Categories table records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         internal Task<DataTable> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression, int startRowIndex, int rows);

         /// <summary>
         /// Selects all Categories
         /// </summary>
         internal Task<DataTable> SelectAllAsync();

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories.
         /// </summary>
         internal Task<DataTable> SelectAllDynamicWhereAsync(int? categoryID, string categoryName);

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control
         /// </summary>
         internal Task<DataTable> SelectCategoryDropDownListDataAsync();

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         internal Task DeleteAsync(int categoryID);

         /// <summary>
         /// Inserts a record
         /// </summary>
         internal Task<int> InsertAsync(Category objCategory);

         /// <summary>
         /// Updates a record
         /// </summary>
         internal Task UpdateAsync(Category objCategory);
     }
}
