using System; 
using System.Text; 

namespace AdHocApi.DataRepository
{ 
    internal sealed class SupplierSql 
    { 
        private SupplierSql() 
        { 
        } 

        internal static string SelectByPrimaryKey() 
        { 
            string selectStatement = GetSelectStatement(); 
            StringBuilder sb = new(); 

            sb.Append(selectStatement);
            sb.Append(" WHERE "); 
            sb.Append("[SupplierID] = @supplierID ");

            return sb.ToString(); 
        } 

        internal static string GetRecordCount() 
        { 
            StringBuilder sb = new(); 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Suppliers]");

            return sb.ToString(); 
        } 

        internal static string GetRecordCountDynamicWhere() 
        { 
            StringBuilder sb = new(); 

            sb.Append("SELECT COUNT(*) AS RecordCount FROM [dbo].[Suppliers]");

            sb.Append(" WHERE ");
            sb.Append("([SupplierID] = @supplierID OR @supplierID IS NULL) AND ");
            sb.Append("([CompanyName] LIKE '%' + @companyName + '%' OR @companyName IS NULL) AND ");
            sb.Append("([ContactName] LIKE '%' + @contactName + '%' OR @contactName IS NULL) AND ");
            sb.Append("([ContactTitle] LIKE '%' + @contactTitle + '%' OR @contactTitle IS NULL) AND ");
            sb.Append("([Address] LIKE '%' + @address + '%' OR @address IS NULL) AND ");
            sb.Append("([City] LIKE '%' + @city + '%' OR @city IS NULL) AND ");
            sb.Append("([Region] LIKE '%' + @region1 + '%' OR @region1 IS NULL) AND ");
            sb.Append("([PostalCode] LIKE '%' + @postalCode + '%' OR @postalCode IS NULL) AND ");
            sb.Append("([Country] LIKE '%' + @country + '%' OR @country IS NULL) AND ");
            sb.Append("([Phone] LIKE '%' + @phone + '%' OR @phone IS NULL) AND ");
            sb.Append("([Fax] LIKE '%' + @fax + '%' OR @fax IS NULL) ");

            return sb.ToString(); 
        } 

        internal static string SelectSkipAndTake() 
        { 
            string selectStatement = GetSelectStatement(); 
            StringBuilder sb = new(); 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start; ");

            sb.Append("SELECT ");
            sb.Append("[SupplierID], ");
            sb.Append("[CompanyName], ");
            sb.Append("[ContactName], ");
            sb.Append("[ContactTitle], ");
            sb.Append("[Address], ");
            sb.Append("[City], ");
            sb.Append("[Region] AS [Region1], ");
            sb.Append("[PostalCode], ");
            sb.Append("[Country], ");
            sb.Append("[Phone], ");
            sb.Append("[Fax], ");
            sb.Append("[HomePage] ");
            sb.Append("FROM [dbo].[Suppliers] ");
            sb.Append("ORDER BY ");
	        sb.Append("CASE WHEN @sortByExpression = 'SupplierID' THEN [SupplierID] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'SupplierID desc' THEN [SupplierID] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName' THEN [CompanyName] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName desc' THEN [CompanyName] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'ContactName' THEN [ContactName] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'ContactName desc' THEN [ContactName] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'ContactTitle' THEN [ContactTitle] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'ContactTitle desc' THEN [ContactTitle] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Address' THEN [Address] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Address desc' THEN [Address] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'City' THEN [City] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'City desc' THEN [City] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Region1' THEN [Region] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Region1 desc' THEN [Region] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode' THEN [PostalCode] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode desc' THEN [PostalCode] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Country' THEN [Country] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Country desc' THEN [Country] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Phone' THEN [Phone] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Phone desc' THEN [Phone] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Fax' THEN [Fax] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Fax desc' THEN [Fax] END DESC ");

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ");
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ");

            return sb.ToString(); 
        } 

        internal static string SelectSkipAndTakeDynamicWhere() 
        { 
            StringBuilder sb = new(); 

            sb.Append("DECLARE @numberOfRowsToSkip int = @start; ");

            sb.Append("SELECT ");
            sb.Append("[SupplierID], ");
            sb.Append("[CompanyName], ");
            sb.Append("[ContactName], ");
            sb.Append("[ContactTitle], ");
            sb.Append("[Address], ");
            sb.Append("[City], ");
            sb.Append("[Region] AS [Region1], ");
            sb.Append("[PostalCode], ");
            sb.Append("[Country], ");
            sb.Append("[Phone], ");
            sb.Append("[Fax], ");
            sb.Append("[HomePage] ");
            sb.Append("FROM [dbo].[Suppliers] ");
            sb.Append(" WHERE ");
            sb.Append("([SupplierID] = @supplierID OR @supplierID IS NULL) AND ");
            sb.Append("([CompanyName] LIKE '%' + @companyName + '%' OR @companyName IS NULL) AND ");
            sb.Append("([ContactName] LIKE '%' + @contactName + '%' OR @contactName IS NULL) AND ");
            sb.Append("([ContactTitle] LIKE '%' + @contactTitle + '%' OR @contactTitle IS NULL) AND ");
            sb.Append("([Address] LIKE '%' + @address + '%' OR @address IS NULL) AND ");
            sb.Append("([City] LIKE '%' + @city + '%' OR @city IS NULL) AND ");
            sb.Append("([Region] LIKE '%' + @region1 + '%' OR @region1 IS NULL) AND ");
            sb.Append("([PostalCode] LIKE '%' + @postalCode + '%' OR @postalCode IS NULL) AND ");
            sb.Append("([Country] LIKE '%' + @country + '%' OR @country IS NULL) AND ");
            sb.Append("([Phone] LIKE '%' + @phone + '%' OR @phone IS NULL) AND ");
            sb.Append("([Fax] LIKE '%' + @fax + '%' OR @fax IS NULL) ");

            sb.Append("ORDER BY ");
	        sb.Append("CASE WHEN @sortByExpression = 'SupplierID' THEN [SupplierID] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'SupplierID desc' THEN [SupplierID] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName' THEN [CompanyName] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'CompanyName desc' THEN [CompanyName] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'ContactName' THEN [ContactName] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'ContactName desc' THEN [ContactName] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'ContactTitle' THEN [ContactTitle] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'ContactTitle desc' THEN [ContactTitle] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Address' THEN [Address] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Address desc' THEN [Address] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'City' THEN [City] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'City desc' THEN [City] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Region1' THEN [Region] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Region1 desc' THEN [Region] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode' THEN [PostalCode] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'PostalCode desc' THEN [PostalCode] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Country' THEN [Country] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Country desc' THEN [Country] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Phone' THEN [Phone] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Phone desc' THEN [Phone] END DESC, ");

	        sb.Append("CASE WHEN @sortByExpression = 'Fax' THEN [Fax] END, ");
	        sb.Append("CASE WHEN @sortByExpression = 'Fax desc' THEN [Fax] END DESC ");

            sb.Append("OFFSET @numberOfRowsToSkip ROWS ");
            sb.Append("FETCH NEXT @numberOfRows ROWS ONLY ");

            return sb.ToString(); 
        } 

        internal static string SelectAll() 
        { 
            string selectStatement = GetSelectStatement(); 
            StringBuilder sb = new(); 

            sb.Append(selectStatement);

            return sb.ToString(); 
        } 

        internal static string SelectAllDynamicWhere()
        {
            string selectStatement = GetSelectStatement();
            StringBuilder sb = new();

            sb.Append(selectStatement);
            sb.Append(" WHERE ");
            sb.Append("([SupplierID] = @supplierID OR @supplierID IS NULL) AND ");
            sb.Append("([CompanyName] LIKE '%' + @companyName + '%' OR @companyName IS NULL) AND ");
            sb.Append("([ContactName] LIKE '%' + @contactName + '%' OR @contactName IS NULL) AND ");
            sb.Append("([ContactTitle] LIKE '%' + @contactTitle + '%' OR @contactTitle IS NULL) AND ");
            sb.Append("([Address] LIKE '%' + @address + '%' OR @address IS NULL) AND ");
            sb.Append("([City] LIKE '%' + @city + '%' OR @city IS NULL) AND ");
            sb.Append("([Region] LIKE '%' + @region1 + '%' OR @region1 IS NULL) AND ");
            sb.Append("([PostalCode] LIKE '%' + @postalCode + '%' OR @postalCode IS NULL) AND ");
            sb.Append("([Country] LIKE '%' + @country + '%' OR @country IS NULL) AND ");
            sb.Append("([Phone] LIKE '%' + @phone + '%' OR @phone IS NULL) AND ");
            sb.Append("([Fax] LIKE '%' + @fax + '%' OR @fax IS NULL) ");

            return sb.ToString();
        } 

        /// <summary>
        /// Selects SupplierID and CompanyName columns for use with a DropDownList web control
        /// </summary>
        internal static string SelectSupplierDropDownListData()
        { 
            string selectStatement = "SELECT [SupplierID], [CompanyName] FROM [dbo].[Suppliers] ORDER BY [CompanyName] ASC "; 
            StringBuilder sb = new(); 

            sb.Append(selectStatement);

            return sb.ToString(); 
        } 

        internal static string Insert()
        { 
            StringBuilder sb = new(); 

            sb.Append("INSERT INTO [dbo].[Suppliers] ");
            sb.Append("(");
            sb.Append("[CompanyName], ");
            sb.Append("[ContactName], ");
            sb.Append("[ContactTitle], ");
            sb.Append("[Address], ");
            sb.Append("[City], ");
            sb.Append("[Region], ");
            sb.Append("[PostalCode], ");
            sb.Append("[Country], ");
            sb.Append("[Phone], ");
            sb.Append("[Fax], ");
            sb.Append("[HomePage] ");
            sb.Append(") ");
            sb.Append("OUTPUT inserted.[SupplierID] ");
            sb.Append("VALUES ");
            sb.Append("(");
            sb.Append("@companyName,");
            sb.Append("@contactName,");
            sb.Append("@contactTitle,");
            sb.Append("@address,");
            sb.Append("@city,");
            sb.Append("@region1,");
            sb.Append("@postalCode,");
            sb.Append("@country,");
            sb.Append("@phone,");
            sb.Append("@fax,");
            sb.Append("@homePage");
            sb.Append(")");

            return sb.ToString(); 
        } 

        internal static string Update()
        { 
            StringBuilder sb = new(); 

            sb.Append("UPDATE [dbo].[Suppliers] ");
            sb.Append("SET ");
            sb.Append("[CompanyName] = @companyName,");
            sb.Append("[ContactName] = @contactName,");
            sb.Append("[ContactTitle] = @contactTitle,");
            sb.Append("[Address] = @address,");
            sb.Append("[City] = @city,");
            sb.Append("[Region] = @region1,");
            sb.Append("[PostalCode] = @postalCode,");
            sb.Append("[Country] = @country,");
            sb.Append("[Phone] = @phone,");
            sb.Append("[Fax] = @fax,");
            sb.Append("[HomePage] = @homePage");
            sb.Append(" WHERE "); 
            sb.Append("[SupplierID] = @supplierID ");

            return sb.ToString(); 
        } 

        internal static string Delete() 
        { 
            StringBuilder sb = new(); 

            sb.Append("DELETE FROM [dbo].[Suppliers] ");
            sb.Append(" WHERE "); 
            sb.Append("[SupplierID] = @supplierID ");

            return sb.ToString(); 
        } 

        private static string GetSelectStatement() 
        { 
            StringBuilder sb = new(); 

            sb.Append("SELECT ");
            sb.Append("[SupplierID], ");
            sb.Append("[CompanyName], ");
            sb.Append("[ContactName], ");
            sb.Append("[ContactTitle], ");
            sb.Append("[Address], ");
            sb.Append("[City], ");
            sb.Append("[Region] AS [Region1], ");
            sb.Append("[PostalCode], ");
            sb.Append("[Country], ");
            sb.Append("[Phone], ");
            sb.Append("[Fax], ");
            sb.Append("[HomePage] ");
            sb.Append("FROM [dbo].[Suppliers] ");

            return sb.ToString(); 
        } 
    } 
} 
